/*
 * 	esm.c - Everest Service Manager
 *
 * 	This file is part of esm.
 *
 * 	Esm is free software: you can redistribute it and/or modify it under the terms of the
 * 	GNU General Public License as published by the Free Software Foundation, either
 * 	version 3 of the License, or (at your option) any later version.
 *
 * 	Esm is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * 	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * 	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License along with Esm. If
 * 	not, see <https://www.gnu.org/licenses/>.
 */

#include <color.h>
#include <getopt.h>
#include <libconfig.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "../release.h"
#include "util.h"

config_t config_context;
config_setting_t *setting;

static void
usage(int argc, char *argv[])
{
	printf(COL_BLUE "[>] " COL_RESET "%s - manage init services\n", argv[0]);
	printf(COL_BLUE "[?] " COL_RESET "usage: %s [-h] [-v] [-s] [-r] [-x]\n", argv[0]);
	printf("\n");
	printf("%s {-h}		Show this message\n", argv[0]);
	printf("%s {-v}		Show the current version\n", argv[0]);
	printf("%s {-s} SRV	Start service SRV\n", argv[0]);
	printf("%s {-r} SRV	Resart service SRV\n", argv[0]);
	printf("%s {-x} SRV	Stop service SRV\n", argv[0]);
	printf("\n");
	printf("Esm is free software.\n");
	printf("See the GNU GPL version 3 for details.\n");
}

static void
usage_small(int argc, char *argv[])
{
	printf(COL_RED "[x] " COL_RESET "usage: %s [-h] [-v] [-s] [-r] [-x] SERVICE_NAME\n", argv[0]);
}

int
main(int argc, char *argv[], char *envp[])
{
	if (argc < 2) {
		printf(COL_RED "[x] " COL_RESET "%s: not enough arguments\n", argv[0]);
		usage_small(argc, argv);
		return EXIT_FAILURE;
	}

	int opt;
	while ((opt = getopt(argc, argv, ":if:hvsrx")) != -1) { 
	/* next line is supposed to be indented but 4 deep indentations aren't exactly pretty */
	/**/switch (opt) {
		case 'h':
			usage(argc, argv);
			return 0;
			break;
		case 'v':
			printf(ESM_RELEASE "\n");
			return 0;
			break;
		case 's':
			config_init(&config_context);

			if (! config_read_file(&config_context, "/etc/esm.cfg")) {
				fprintf(stderr, COL_RED "[x] " COL_RESET "%s: failed to open /etc/esm.cfg: No such file or directory\n", argv[0]);
				config_destroy(&config_context);
				return EXIT_FAILURE;
			}

			if (argc < 3) {
				printf(COL_RED "[x] " COL_RESET "%s: no service specified\n", argv[0]);
				usage_small(argc, argv);
				return EXIT_FAILURE;
			}

			printf(COL_BLUE "[i] " COL_RESET "Starting service %s...\n", argv[2]);

			char *srv = argv[2];

			const char *SRV_ARGS[] = {}
			const char *SRV_ENV[] = {}

			execve();

			return 0;
			break;
		case 'r':
			config_init(&config_context);

			if (! config_read_file(&config_context, "/etc/esm.cfg")) {
				fprintf(stderr, COL_RED "[x] " COL_RESET "%s: failed to open /etc/esm.cfg: No such file or directory\n", argv[0]);
				config_destroy(&config_context);
				return EXIT_FAILURE;
			}
			return 0;
			break;
		case 'x':
			config_init(&config_context);

			if (! config_read_file(&config_context, "/etc/esm.cfg")) {
				fprintf(stderr, COL_RED "[x] " COL_RESET "%s: failed to open /etc/esm.cfg: No such file or directory\n", argv[0]);
				config_destroy(&config_context);
				return EXIT_FAILURE;
			}
			return 0;
			break;
		case '?':
			fprintf(stderr, COL_RED "[x] " COL_RESET "%s: unknown argument\n", argv[0]);
			usage_small(argc, argv);
			return 1;
			break;
		default:
			fprintf(stderr, COL_RED "[x] " COL_RESET "%s: unknown argument\n", argv[0]);
			usage_small(argc, argv);
			return 1;
			break;
	}}

	fprintf(stderr, COL_RED "[x] " COL_RESET "%s: unknown argument\n", argv[0]);
	usage_small(argc, argv);
	return 1;
}
